import type { INode, WorkflowParameters } from 'n8n-workflow';
import { NodeConnectionType, Workflow } from 'n8n-workflow';
export type GraphConnection = {
    from: INode;
    to: INode;
    type: NodeConnectionType;
    outputIndex: number;
    inputIndex: number;
};
export declare class DirectedGraph {
    private nodes;
    private connections;
    getNodes(): Map<string, INode>;
    getConnections(filter?: {
        to?: INode;
    }): GraphConnection[];
    addNode(node: INode): this;
    addNodes(...nodes: INode[]): this;
    addConnection(connectionInput: {
        from: INode;
        to: INode;
        type?: NodeConnectionType;
        outputIndex?: number;
        inputIndex?: number;
    }): this;
    addConnections(...connectionInputs: Array<{
        from: INode;
        to: INode;
        type?: NodeConnectionType;
        outputIndex?: number;
        inputIndex?: number;
    }>): this;
    getDirectChildren(node: INode): GraphConnection[];
    private getChildrenRecursive;
    getChildren(node: INode): Set<INode>;
    getDirectParents(node: INode): GraphConnection[];
    getConnection(from: INode, outputIndex: number, type: NodeConnectionType, inputIndex: number, to: INode): GraphConnection | undefined;
    toWorkflow(parameters: Omit<WorkflowParameters, 'nodes' | 'connections'>): Workflow;
    static fromWorkflow(workflow: Workflow): DirectedGraph;
    private toIConnections;
    private makeKey;
}
