"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findSubgraph = findSubgraph;
const DirectedGraph_1 = require("./DirectedGraph");
function findSubgraphRecursive(graph, destinationNode, current, trigger, newGraph, currentBranch) {
    if (current === trigger) {
        for (const connection of currentBranch) {
            newGraph.addNodes(connection.from, connection.to);
            newGraph.addConnection(connection);
        }
        return;
    }
    let parentConnections = graph.getDirectParents(current);
    if (parentConnections.length === 0) {
        return;
    }
    const isCycleWithDestinationNode = current === destinationNode && currentBranch.some((c) => c.to === destinationNode);
    if (isCycleWithDestinationNode) {
        return;
    }
    const isCycleWithCurrentNode = currentBranch.some((c) => c.to === current);
    if (isCycleWithCurrentNode) {
        for (const connection of currentBranch) {
            newGraph.addNodes(connection.from, connection.to);
            newGraph.addConnection(connection);
        }
        return;
    }
    if (current.disabled) {
        const incomingConnections = graph.getDirectParents(current);
        const outgoingConnections = graph
            .getDirectChildren(current)
            .filter((connection) => connection.outputIndex === 0);
        parentConnections = [];
        for (const incomingConnection of incomingConnections) {
            for (const outgoingConnection of outgoingConnections) {
                const newConnection = {
                    ...incomingConnection,
                    to: outgoingConnection.to,
                    inputIndex: outgoingConnection.inputIndex,
                };
                parentConnections.push(newConnection);
                currentBranch.pop();
                currentBranch.push(newConnection);
            }
        }
    }
    for (const parentConnection of parentConnections) {
        findSubgraphRecursive(graph, destinationNode, parentConnection.from, trigger, newGraph, [
            ...currentBranch,
            parentConnection,
        ]);
    }
}
function findSubgraph(graph, destinationNode, trigger) {
    const newGraph = new DirectedGraph_1.DirectedGraph();
    findSubgraphRecursive(graph, destinationNode, destinationNode, trigger, newGraph, []);
    return newGraph;
}
//# sourceMappingURL=findSubgraph.js.map